==> endeavour_2.7.3.orig.tar.gz: endeavour-2.7.3/endeavour2/edv_pixmap.c <==
#include <string.h>
#include <gtk/gtk.h>
#include "guiutils.h"
#include "edv_pixmap.h"


#ifndef DEBUG_LEVEL
# define DEBUG_LEVEL	0
#endif


GList *EDVPixmapListNew(
	GList *pixmaps_list,
	guint8 **data, const gchar *name,
	edv_pixmap **p_rtn
);
GList *EDVPixmapListClean(GList *pixmaps_list);

edv_pixmap *EDVPixmapListMatchByName(
	GList *glist, const gchar *name  
);

edv_pixmap *EDVPixmapNew(guint8 **data, const gchar *name);
void EDVPixmapDelete(edv_pixmap *p);


#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)


/*
 *	Creates a new Pixmap as needed or returns one that matches
 *	the specified values from the list.
 */
GList *EDVPixmapListNew(
	GList *pixmaps_list,
	guint8 **data, const gchar *name,
	edv_pixmap **p_rtn
)
{
	/* Check if pixmap already exists with the specified name */
	edv_pixmap *p = EDVPixmapListMatchByName(pixmaps_list, name);
	if(p != NULL)
	{
#if (DEBUG_LEVEL >= 1)  
 g_print(
  "EDVPixmapListNew(): Returning existing edv_pixmap \"%s\" with %i ref counts\n",
  name,
  gdk_window_get_ref_count(p->pixmap)
 );
#endif
	    if(p_rtn != NULL)
		*p_rtn = p;
	    return(pixmaps_list);
	}
	else
	{
	    GList *glist = pixmaps_list;

	    /* Need to create a new pixmap */
	    p = EDVPixmapNew(data, name);

	    /* Add to the list, look for an available pointer first,
	     * if none are available then append to the list
	     */	    
	    while(glist != NULL)
	    {
		if(glist->data == NULL)
		    break;
		glist = g_list_next(glist);
	    }
	    /* Found available pointer? */
	    if(glist != NULL)
		glist->data = p;
	    else
		pixmaps_list = g_list_append(pixmaps_list, p);

#if (DEBUG_LEVEL >= 1)
 g_print(
  "EDVPixmapListNew(): Creating new edv_pixmap \"%s\"\n",
  name                                        
 );   
#endif
	    if(p_rtn != NULL)
		*p_rtn = p;
	    return(pixmaps_list);
	}
}

/*
 *	Deletes all the pixmaps in the list that have only 1 reference
 *	count remaining.
 */
GList *EDVPixmapListClean(GList *pixmaps_list)
{
	edv_pixmap *p;
	GList *glist = pixmaps_list;

#if (DEBUG_LEVEL >= 3)
 g_print(
  "EDVPixmapListClean(): Cleaning edv_pixmaps...\n"
 );
#endif

	/* Iterate through the list and delete all the pixmaps that
	 * have only 1 ref count or less
	 */
	while(glist != NULL)
	{
	    p = EDV_PIXMAP(glist->data);
	    if(p != NULL)
	    {
#if (DEBUG_LEVEL >= 3)
 g_print(
  "EDVPixmapListClean(): Checking edv_pixmap \"%s\"\n",
  p->name
 );
#endif
		/* This pixmap only has one ref count left? */
		if(gdk_window_get_ref_count(p->pixmap) <= 1)
		{
#if (DEBUG_LEVEL >= 2)
 g_print(
  "EDVPixmapListClean(): Found edv_pixmap \"%s\" with 1 or less ref counts remaining\n",
  p->name
 );
#endif
		    /* Delete this pixmap and remove it from the
		     * list then start the search from the beginning
		     * of the list again
		     */
		    EDVPixmapDelete(p);
		    glist = pixmaps_list = g_list_remove(pixmaps_list, p);
		    continue;
		}
	    }
	    glist = g_list_next(glist);
	}
	return(pixmaps_list);
}


/*
 *	Returns a pixmap from the list who's name matches the specified
 *	the specified name.
 */
edv_pixmap *EDVPixmapListMatchByName(
	GList *pixmaps_list, const gchar *name
)
{
	GList *glist = pixmaps_list;
	edv_pixmap *p;

	if(STRISEMPTY(name))
	    return(NULL);

#if (DEBUG_LEVEL >= 3)
 g_print(
  "EDVPixmapListMatchByName(): Searching for edv_pixmap \"%s\"...\n",
  >name
 );
#endif

	while(glist != NULL)
	{
	    p = EDV_PIXMAP(glist->data);
	    if(p != NULL)
	    {
#if (DEBUG_LEVEL >= 3)
 g_print(
  "EDVPixmapListMatchByName(): Checking edv_pixmap \"%s\"\n",
  p->name
 );
#endif
		if((p->name != NULL) ? !strcmp(p->name, name) : FALSE)
		{
#if (DEBUG_LEVEL >= 2)
 g_print(
  "EDVPixmapListMatchByName(): Matched edv_pixmap \"%s\"\n",
  p->name
 );
#endif
		    return(p);
		}
	    }
	    glist = g_list_next(glist);
	}

	return(NULL);
}



/*
 *	Creates a new Pixmap from the specified XPM data.
 */
edv_pixmap *EDVPixmapNew(guint8 **data, const gchar *name)
{
	edv_pixmap *p = EDV_PIXMAP(g_malloc0(
	    sizeof(edv_pixmap)
	));

	/* XPM data given? */
	if(data != NULL)
	{
	    GdkPixmap *pixmap;

	    /* Create pixmap and mask pair from the XPM data */
	    p->pixmap = pixmap = GDK_PIXMAP_NEW_FROM_XPM_DATA(
		&p->mask, data
	    );
	    if(pixmap != NULL)
	    {
		gdk_window_get_size(pixmap, &p->width, &p->height);
		p->data = data;
	    }
	}

	/* Set name */
	p->name = STRDUP(name);
	if(STRISEMPTY(p->name))
	    g_printerr(
"EDVPixmapNew(): Warning:\
 Creating edv_pixmap 0x%.8x from data 0x%.8x with no name.\n",
		(guint32)p, (guint32)data
	    );

	/* failed to load pixmap? */
	if((data != NULL) && (p->pixmap == NULL))
	    g_printerr(
"EDVPixmapNew(): Warning:\
 Failed to load GdkPixmap for edv_pixmap \"%s\" from data 0x%.8x.\n",
		p->name, (guint32)data
	    );


	return(p);
}

/*
 *	Deletes the Pixmap.
 */
void EDVPixmapDelete(edv_pixmap *p)
{
	if(p == NULL)
	    return;

#if (DEBUG_LEVEL >= 1)
 g_print(
  "EDVPixmapDelete(): Deleting edv_pixmap \"%s\"\n",
  p->name
 );
#endif

	if(p->pixmap != NULL)
	{
	    const gint ref_count = gdk_window_get_ref_count(p->pixmap);
	    if(ref_count > 1)
		g_printerr(
"EDVPixmapDelete(): Warning:\
 Deleting edv_pixmap \"%s\" with %i reference counts.\n",
		    p->name, ref_count
		);
	}

	GDK_PIXMAP_UNREF(p->pixmap)
	GDK_BITMAP_UNREF(p->mask)
	g_free(p->name);
	g_free(p);
}

==> endeavour_2.8.4.orig.tar.gz: endeavour-2.8.4/endeavour2/edv_pixmap.c <==
#include <string.h>
#include <gtk/gtk.h>
#include "guiutils.h"
#include "edv_pixmap.h"


#ifndef DEBUG_LEVEL
# define DEBUG_LEVEL	0
#endif


GList *EDVPixmapListNew(
	GList *pixmaps_list,
	guint8 **data, const gchar *name,
	edv_pixmap **p_rtn
);
GList *EDVPixmapListClean(GList *pixmaps_list);

edv_pixmap *EDVPixmapListMatchByName(
	GList *glist, const gchar *name  
);

edv_pixmap *EDVPixmapNew(guint8 **data, const gchar *name);
void EDVPixmapDelete(edv_pixmap *p);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)


/*
 *	Creates a new Pixmap as needed or returns one that matches
 *	the specified values from the list.
 */
GList *EDVPixmapListNew(
	GList *pixmaps_list,
	guint8 **data, const gchar *name,
	edv_pixmap **p_rtn
)
{
	/* Check if pixmap already exists with the specified name */
	edv_pixmap *p = EDVPixmapListMatchByName(pixmaps_list, name);
	if(p != NULL)
	{
#if (DEBUG_LEVEL >= 1)  
 g_print(
  "EDVPixmapListNew(): Returning existing edv_pixmap \"%s\" with %i ref counts\n",
  name,
  gdk_window_get_ref_count(p->pixmap)
 );
#endif
	    if(p_rtn != NULL)
		*p_rtn = p;
	    return(pixmaps_list);
	}
	else
	{
	    GList *glist = pixmaps_list;

	    /* Need to create a new pixmap */
	    p = EDVPixmapNew(data, name);

	    /* Add to the list, look for an available pointer first,
	     * if none are available then append to the list
	     */	    
	    while(glist != NULL)
	    {
		if(glist->data == NULL)
		    break;
		glist = g_list_next(glist);
	    }
	    /* Found available pointer? */
	    if(glist != NULL)
		glist->data = p;
	    else
		pixmaps_list = g_list_append(pixmaps_list, p);

#if (DEBUG_LEVEL >= 1)
 g_print(
  "EDVPixmapListNew(): Creating new edv_pixmap \"%s\"\n",
  name                                        
 );   
#endif
	    if(p_rtn != NULL)
		*p_rtn = p;
	    return(pixmaps_list);
	}
}

/*
 *	Deletes all the pixmaps in the list that have only 1 reference
 *	count remaining.
 */
GList *EDVPixmapListClean(GList *pixmaps_list)
{
	edv_pixmap *p;
	GList *glist = pixmaps_list;

#if (DEBUG_LEVEL >= 3)
 g_print(
  "EDVPixmapListClean(): Cleaning edv_pixmaps...\n"
 );
#endif

	/* Iterate through the list and delete all the pixmaps that
	 * have only 1 ref count or less
	 */
	while(glist != NULL)
	{
	    p = EDV_PIXMAP(glist->data);
	    if(p != NULL)
	    {
#if (DEBUG_LEVEL >= 3)
 g_print(
  "EDVPixmapListClean(): Checking edv_pixmap \"%s\"\n",
  p->name
 );
#endif
		/* This pixmap only has one ref count left? */
		if(gdk_window_get_ref_count(p->pixmap) <= 1)
		{
#if (DEBUG_LEVEL >= 2)
 g_print(
  "EDVPixmapListClean(): Found edv_pixmap \"%s\" with 1 or less ref counts remaining\n",
  p->name
 );
#endif
		    /* Delete this pixmap and remove it from the
		     * list then start the search from the beginning
		     * of the list again
		     */
		    EDVPixmapDelete(p);
		    glist = pixmaps_list = g_list_remove(pixmaps_list, p);
		    continue;
		}
	    }
	    glist = g_list_next(glist);
	}
	return(pixmaps_list);
}


/*
 *	Returns a pixmap from the list who's name matches the specified
 *	the specified name.
 */
edv_pixmap *EDVPixmapListMatchByName(
	GList *pixmaps_list, const gchar *name
)
{
	GList *glist = pixmaps_list;
	edv_pixmap *p;

	if(STRISEMPTY(name))
	    return(NULL);

#if (DEBUG_LEVEL >= 3)
 g_print(
  "EDVPixmapListMatchByName(): Searching for edv_pixmap \"%s\"...\n",
  >name
 );
#endif

	while(glist != NULL)
	{
	    p = EDV_PIXMAP(glist->data);
	    if(p != NULL)
	    {
#if (DEBUG_LEVEL >= 3)
 g_print(
  "EDVPixmapListMatchByName(): Checking edv_pixmap \"%s\"\n",
  p->name
 );
#endif
		if((p->name != NULL) ? !strcmp(p->name, name) : FALSE)
		{
#if (DEBUG_LEVEL >= 2)
 g_print(
  "EDVPixmapListMatchByName(): Matched edv_pixmap \"%s\"\n",
  p->name
 );
#endif
		    return(p);
		}
	    }
	    glist = g_list_next(glist);
	}

	return(NULL);
}



/*
 *	Creates a new Pixmap from the specified XPM data.
 */
edv_pixmap *EDVPixmapNew(guint8 **data, const gchar *name)
{
	edv_pixmap *p = EDV_PIXMAP(g_malloc0(
	    sizeof(edv_pixmap)
	));

	/* XPM data given? */
	if(data != NULL)
	{
	    GdkPixmap *pixmap;

	    /* Create pixmap and mask pair from the XPM data */
	    p->pixmap = pixmap = GDK_PIXMAP_NEW_FROM_XPM_DATA(
		&p->mask, data
	    );
	    if(pixmap != NULL)
	    {
		gdk_window_get_size(pixmap, &p->width, &p->height);
		p->data = data;
	    }
	}

	/* Set name */
	p->name = STRDUP(name);
	if(STRISEMPTY(p->name))
	    g_printerr(
"EDVPixmapNew(): Warning:\
 Creating edv_pixmap 0x%.8x from data 0x%.8x with no name.\n",
		(guint32)p, (guint32)data
	    );

	/* failed to load pixmap? */
	if((data != NULL) && (p->pixmap == NULL))
	    g_printerr(
"EDVPixmapNew(): Warning:\
 Failed to load GdkPixmap for edv_pixmap \"%s\" from data 0x%.8x.\n",
		p->name, (guint32)data
	    );


	return(p);
}

/*
 *	Deletes the Pixmap.
 */
void EDVPixmapDelete(edv_pixmap *p)
{
	if(p == NULL)
	    return;

#if (DEBUG_LEVEL >= 1)
 g_print(
  "EDVPixmapDelete(): Deleting edv_pixmap \"%s\"\n",
  p->name
 );
#endif

	if(p->pixmap != NULL)
	{
	    const gint ref_count = gdk_window_get_ref_count(p->pixmap);
	    if(ref_count > 1)
		g_printerr(
"EDVPixmapDelete(): Warning:\
 Deleting edv_pixmap \"%s\" with %i reference %s remaining.\n",
		    p->name,
		    ref_count,
		    (ref_count == 1) ? "count" : "counts"
		);
	}

	GDK_PIXMAP_UNREF(p->pixmap);
	GDK_BITMAP_UNREF(p->mask);
	g_free(p->name);
	g_free(p);
}
